/*
 *  The Mana Client
 *  Copyright (C) 2004-2009  The Mana World Development Team
 *  Copyright (C) 2009-2010  The Mana Developers
 *
 *  This file is part of The Mana Client.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "flooritemmanager.h"
#include "flooritem.h"
#include "localplayer.h"
#include "configuration.h"

#include "game.h"

#include "utils/dtor.h"

FloorItemManager::~FloorItemManager()
{
    clear();
}

FloorItem *FloorItemManager::create(int id, int itemId, int x, int y)
{
    Map *map = Game::instance()->getCurrentMap();
    FloorItem *floorItem = new FloorItem(id, itemId, x, y, map);
    mFloorItems.push_back(floorItem);
    return floorItem;
}

void FloorItemManager::destroy(FloorItem *item)
{
    mFloorItems.remove(item);
    delete item;
    item = 0;
}

void FloorItemManager::clear()
{
    delete_all(mFloorItems);
    mFloorItems.clear();
}

FloorItem *FloorItemManager::findById(int id) const
{
    FloorItems::const_iterator i;
    for (i = mFloorItems.begin(); i != mFloorItems.end(); i++)
    {
        if ((*i)->getId() == id)
            return *i;
    }

    return 0;
}

FloorItem *FloorItemManager::findByCoordinates(int x, int y) const
{
    FloorItems::const_iterator i;
    for (i = mFloorItems.begin(); i != mFloorItems.end(); i++)
    {
        if ((*i)->getX() == x && (*i)->getY() == y)
        {
            return *i;
        }
    }

    return NULL;
}

/*
void FloorItemManager::pickUpAll(int x, int y)
{
    FloorItemIterator i;
    for (i = mFloorItems.begin(); i != mFloorItems.end(); i++)
    {
        if (((*i)->getX() >= x - 1 && (*i)->getX() <= x + 1)
            && ((*i)->getY() >= y - 1 && (*i)->getY() <= y + 1))
        {
            player_node->pickUp(*i);
        }
    }

}
*/

bool FloorItemManager::pickUpAll(int x1, int y1, int x2, int y2)
{
    bool finded(false);
    FloorItemIterator i;
    for (i = mFloorItems.begin(); i != mFloorItems.end(); i++)
    {
        if (((*i)->getX() >= x1 && (*i)->getX() <= x2)
            && ((*i)->getY() >= y1 && (*i)->getY() <= y2))
        {
            if (player_node->pickUp(*i))
                finded = true;
        }
    }
    return finded;
}

bool FloorItemManager::pickUpNearest(int x, int y, int maxdist)
{
    maxdist = maxdist * maxdist;
    FloorItem *closestItem = NULL;
    int dist = 0;

    FloorItemIterator i;
    for (i = mFloorItems.begin(); i != mFloorItems.end(); i++)
    {
        FloorItem *item = *i;

        int d = (item->getX() - x) * (item->getX() - x)
                + (item->getY() - y) * (item->getY() - y);

        if ((d < dist || closestItem == NULL) &&
            (!config.getValueTargetOnlyReachable()
            || player_node->isReachable(item->getX(), item->getY())))
        {
            dist = d;
            closestItem = item;
        }
    }
    if (closestItem && dist <= maxdist)
    {
        return player_node->pickUp(closestItem);
//        return true;
    }
    return false;
}
