/*
 *  The Mana World
 *  Copyright (C) 2009  The Mana World Development Team
 *  Copyright (C) 2009-2010  Andrei Karas
 *
 *  This file is part of The Mana World.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef TEXTCOMMAND_H
#define TEXTCOMMAND_H

#include "resources/itemdb.h"

#define MAGIC_START_ID 340

enum SpellTarget
{
    NOTARGET = 0,
    ALLOWTARGET = 1,
    NEEDTARGET = 2
};

enum MagicSchool
{
    SKILL_MAGIC = 340,
    SKILL_MAGIC_LIFE = 341,
    SKILL_MAGIC_WAR = 342,
    SKILL_MAGIC_TRANSMUTE = 343,
    SKILL_MAGIC_NATURE = 344,
    SKILL_MAGIC_ASTRAL = 345
};

enum TextCommandType
{
    TEXT_COMMAND_MAGIC = 0,
    TEXT_COMMAND_TEXT = 1
};

/**
 * Represents one or more instances of a certain item type.
 */
class TextCommand
{
    public:
        /**
         * Constructor.
         */
        TextCommand(unsigned int id, std::string symbol,
                    std::string command, SpellTarget type,
                    unsigned int basicLvl, MagicSchool school = SKILL_MAGIC,
                    unsigned int schoolLvl = 0, int mana = 0);

        /**
         * Constructor.
         */
        TextCommand(unsigned int id, std::string symbol, std::string command, SpellTarget type);

        /**
         * Constructor.
         */
        TextCommand(unsigned int id);

        /**
         * Destructor.
         */
        ~TextCommand();

        std::string getName() {return mCommand; }
        std::string getCommand() {return mCommand; }
        std::string getSymbol() {return mSymbol; }
        unsigned int getId() {return mId; }
        SpellTarget getTargetType() {return mTargetType; }
        int getMana() {return mMana; }
        MagicSchool getSchool() {return mSchool; }
        unsigned int getBaseLvl() {return mBaseLvl; }
        unsigned int getSchoolLvl() {return mSchoolLvl; }
        TextCommandType getCommandType() {return mCommandType; }

        void setCommand(std::string command) {mCommand = command; }
        void setSymbol(std::string symbol) {mSymbol = symbol; }
        void setId(unsigned int id) {mId = id; }
        void setTargetType(SpellTarget targetType) {mTargetType = targetType; }
        void setMana(unsigned int mana) {mMana = mana; }
        void setSchool(MagicSchool school) {mSchool = school; }
        void setBaseLvl(unsigned int baseLvl) {mBaseLvl = baseLvl; }
        void setSchoolLvl(unsigned int schoolLvl) {mSchoolLvl = schoolLvl; }
        void setCommandType(TextCommandType commandType) {mCommandType = commandType; }
        bool isEmpty() {return mCommand == "" && mSymbol == "" ; }

    protected:
        std::string mCommand;
        std::string mSymbol;
        SpellTarget mTargetType;
        unsigned int mId;
        int mMana;
        MagicSchool mSchool;
        unsigned int mBaseLvl;
        unsigned int mSchoolLvl;
        TextCommandType mCommandType;
};

#endif
